/***************************************************************************
                          kspellinfo.cpp  -  description
                             -------------------
    begin                : Sun May 12 2002
    copyright            : (C) 2002 by Zach Hobbs
    email                : zach@firstwavedesign.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kspellinfo.h"

KSpellInfo::KSpellInfo( QWidget* parent=0, const char *name=0 )
  : QWidget( parent, name ){

  getDetails = 0;

  //Layouts
  QVBoxLayout *vertical = new QVBoxLayout( this );
  
  QHBoxLayout *row1 = new QHBoxLayout();
  QHBoxLayout *row2 = new QHBoxLayout();
  QHBoxLayout *row3 = new QHBoxLayout();
  QHBoxLayout *row4 = new QHBoxLayout();
  QHBoxLayout *row5 = new QHBoxLayout();
  QHBoxLayout *row6 = new QHBoxLayout();
  QVBoxLayout *row7 = new QVBoxLayout();
  QHBoxLayout *actionRow = new QHBoxLayout();

  vertical->addLayout( row1 );
  vertical->addLayout( row2 );  
  vertical->addLayout( row3 ); 
  vertical->addLayout( row4 ); 
  vertical->addLayout( row5 ); 
  vertical->addLayout( row6 ); 
  vertical->addLayout( row7 );
  vertical->addLayout( actionRow );

  vertical->setMargin( 20 );
  vertical->setSpacing( 20 );
 
  //row 1
  QLabel* spellnamelabel = new QLabel( "Spell Name:", this );
  spellnameline = new QLabel( "Select A Spell", this );

  row1->addWidget( spellnamelabel );
  row1->addWidget( spellnameline );

  spellnamelabel->show();
  spellnameline->show();

  //row 2
  QLabel* spellversionlabel = new QLabel( "Version: ", this );
  spellversionline = new QLabel( this );
  
  row2->addWidget( spellversionlabel );
  row2->addWidget( spellversionline );

  spellversionlabel->show();
  spellversionline->show();

  //row 3
  QLabel* spellwebsitelabel = new QLabel( "Website: ", this );
  spellwebsiteline = new KURLLabel( this );
  
  row3->addWidget( spellwebsitelabel );
  row3->addWidget( spellwebsiteline );

  spellwebsitelabel->show();
  spellwebsiteline->show();
  
  //row 4
  QLabel* spellsourcelabel = new QLabel( "Source: ", this );
  spellsourceline = new QLabel( this );
  
  row4->addWidget( spellsourcelabel );
  row4->addWidget( spellsourceline );

  spellsourcelabel->show();
  spellsourceline->show();

  //row 5
  QLabel* spelllicenselabel = new QLabel( "License: ", this );
  spelllicenseline = new QLabel( this );
  
  row5->addWidget( spelllicenselabel );
  row5->addWidget( spelllicenseline );

  spelllicenselabel->show();
  spelllicenseline->show();

  //row 6
  QLabel* spellmaintainerlabel = new QLabel( "Maintainer: ", this );
  spellmaintainerline = new KURLLabel( this );
  
  row6->addWidget( spellmaintainerlabel );
  row6->addWidget( spellmaintainerline );

  spellmaintainerlabel->show();
  spellmaintainerline->show();

  //row7
  spelldescriptionlabel = new QLabel( "Spell Description:", this );
  spelldescriptionline = new QTextEdit( this, "Spell Description" );
  spelldescriptionline->setReadOnly( true );
  spelldescriptionline->setMinimumWidth( 450 );
  spelldescriptionline->setMinimumHeight( 200 );

  row7->addWidget( spelldescriptionlabel );
  row7->addWidget( spelldescriptionline );

  spelldescriptionlabel->show();
  spelldescriptionline->show();
  
  //actionRow
  actions = new QComboBox( false, this );
  execAction = new QPushButton( "Execute", this );

  actions->insertItem("-Select Action-");
  
  actionRow->addWidget( actions );
  actionRow->addWidget( execAction );

  //Connections
  //Action button
  QObject::connect( execAction, SIGNAL( clicked() ),
		    this, SLOT( actionExecClicked() ) );

  QObject::connect( spellwebsiteline, SIGNAL(leftClickedURL(const QString&)),
		    this, SLOT(processMyURL(const QString&)));

}

KSpellInfo::~KSpellInfo(){

}

void KSpellInfo::updateInfo( QListViewItem * spell ) {

  if (spell->depth() == 2) {

    //Get more Details
    QString spellname = spell->text(0);
    QString section = spell->parent()->text(0);
    QString cmd;
    
    // Humm, process still running from hasty users?
    if (getDetails != 0) {
      getDetails->kill();
      delete getDetails;
      qDebug("You're too hasty.");
    }
    
    getDetails = new QProcess( this );
    this->getDetails->addArgument("bash");
    
    cmd = ". " + spell->parent()->text(4) + "/" + spellname +"/DETAILS";
    cmd += "&& echo -e ' \\nnext\\n'$VERSION' \\nnext\\n'$WEB_SITE' \\nnext\\n'$LICENSE' \\nnext\\n'$SOURCE' \\nnext\\n'$MAINTAINER' \\nnext\\n'";
    
    if ( !getDetails->launch( cmd ) )
      qDebug("won't start:" + getDetails->arguments().join(" ") + "\n" + QString(getDetails->readStderr()));

    QObject::connect( getDetails, SIGNAL( processExited() ),
		     this, SLOT( detailsReady() ) );
    
    spellnameline->setText( spellname );
    spelldescriptionlabel->setText( "Spell Description for " + spellname + ":");
    //Determain what actions available
    
    actions->clear();
    actions->insertItem( "- Select Action -");

    if ( spell->text(3) == "installed" || spell->text(3) == "held" ) {
      actions->insertItem( "Dispel Spell" );      
      actions->insertItem( "Cast -r -c Spell" );
      
      if ( spell->text(2) != spell->text(1) ) {

	actions->insertItem( "Summon Spell" );
      }
      
    } else {
      actions->insertItem( "Add to queue" );
      actions->insertItem( "Cast Spell" );
      actions->insertItem( "Summon Spell" );
    }

  }
}

void KSpellInfo::detailsReady() {
  QByteArray spelldetails;
  QStringList detailsLines;

  spelldetails = getDetails->readStdout();

  detailsLines = QStringList::split( "\nnext\n", QString(spelldetails) );
  
  QStringList::Iterator it = detailsLines.begin();
  spelldescriptionline->setText( *it );
  
  if ( it != detailsLines.end() ) {
    it++;
    spellversionline->setText( *it );
  }

  if ( it != detailsLines.end() ) {
    it++;
    if (*it != ' ')
      spellwebsiteline->setURL( QString(*it).simplifyWhiteSpace() );
    spellwebsiteline->setText( QString(*it).simplifyWhiteSpace() );
  }

  if ( it != detailsLines.end() ) {
    it++;
    spelllicenseline->setText( *it );
  }

  if ( it != detailsLines.end() ) {
    it++;
    spellsourceline->setText( *it );
  }

  if ( it != detailsLines.end() ) {
    it++;
    spellmaintainerline->setText( QString(*it).simplifyWhiteSpace() );
    if (*it != ' ')
      spellmaintainerline->setURL( "mailto:" + QString(*it).simplifyWhiteSpace() );
  }

  getDetails->clearArguments();
  delete getDetails;
  getDetails = 0;
}

void KSpellInfo::actionExecClicked() {

  if ( actions->currentItem() == 0 )
    return;
  else if ( actions->currentText() == "Add to queue" ) {
    QString * spellname = new QString(spellnameline->text());
    emit addQueue( spellname );

  } else {
    QString* cmd;
  
    cmd = new QString( actions->currentText() );
    *cmd = cmd->lower();
    *cmd->remove( "spell" );
    *cmd += spellnameline->text();
    
    emit actionTaken( cmd );
  }

}

void KSpellInfo::processMyURL( const QString &URL ) {

  KRun *startURL = new KRun( URL, 0, false, true );
  

}
